#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/twi.h>


#define TWI_flags		EEDR

//  
#define AllModes		0x03
#define WaitMode		0x00
#define AlarmMode		0x01

//  TWI
#define I2cError		0x00
#define SendFrame		0x01
#define ReciveFrame		0x02
#define SendStart		0x03
#define SendAdres		0x04
#define SendData		0x05


#define GreenLedOn() PORTE |= 0x08
#define GreenLedOff() PORTE &=~ 0x08
#define GreenLedTogle() PORTE ^= 0x08

#define RedLedOn() PORTE |= 0x20
#define RedLedOff() PORTE &=~ 0x20
#define RedLedTogle() PORTE ^= 0x20

#define OrangeLedOn() PORTE |= 0x28
#define OrangeOff() PORTE &=~ 0x28
#define OrangeTogle() PORTE ^= 0x28

#define Relay1On() PORTE |= 0x10
#define Relay1Off() PORTE &=~ 0x10

#define SensorPowerOn() PORTB |= 0x01
#define SensorPowerOff() PORTB &=~ 0x01


volatile unsigned char I2cFrameLen;
volatile unsigned char *I2cFrame;
unsigned char I2cBufer[16];


ISR(TWI_vect)										//  TWI
{
 register unsigned char status;
 
 status = TWSR & 0xF8;

 if(TWI_flags & _BV(SendFrame))				//    
 {
  switch (status)							//   TWI
  {
   case TW_START:							//   ""
   {
	if(TWI_flags & _BV(SendStart))			//   ""
	{
     TWI_flags &=~_BV(SendStart);			//     
	 TWI_flags |= _BV(SendAdres);			//     	
	 TWDR = *I2cFrame++;					//  : 7   0 -  
  	 TWCR=(1<<TWINT)|(1<<TWEN)|(1<<TWIE);	//    TWDR   
	 return;								//    ,   
	}//if(TWI_flags & _BV(SendStart))
	break;									//    
   }//case TW_START

   case TW_MT_SLA_ACK:						//      
   {
 	if(TWI_flags & _BV(SendAdres))			//    
	{
     TWI_flags &=~_BV(SendAdres);			//    
	 TWI_flags |= _BV(SendData);			//   
	 TWDR = *I2cFrame++;					//   
  	 TWCR=(1<<TWINT)|(1<<TWEN)|(1<<TWIE);	//    TWDR   
	 return;								//    ,   
	}//if(TWI_flags & _BV(SendAdres))
	break;									//     
   }//case TW_MT_SLA_ACK:

   case TW_MT_DATA_ACK:						//      
   {
 	if(TWI_flags & _BV(SendData))			//    
	{
     I2cFrameLen--;							//   
	 if(I2cFrameLen)						//    
	 {
	  TWDR = *I2cFrame++;					//   
  	  TWCR=(1<<TWINT)|(1<<TWEN)|(1<<TWIE);	//    TWDR   
	 }//if(I2cFrameLen)
	 else									//      
	 {
	  TWI_flags &=~_BV(SendData);			//      
	  TWI_flags &=~_BV(SendFrame);			//   
 	  TWCR=(1<<TWINT)|(1<<TWSTO)|(1<<TWEN);	//  
	 }//else
	 return;								//    ,   
	}//if(TWI_flags & _BV(SendData))
	break;									//     
   }//case TW_MT_DATA_ACK:
  }//switch (status)
  TWI_flags = _BV(I2cError);				//    TWI
 }//if(bit_is_set(TWI_flags,SendFrame))
}


void init_devices(void)				//  
{
 cli();

 //  
 PORTF = 0b00000000;
 DDRF =  0b00000000;

 PORTE = 0b01000011;
 DDRE =  0b00111100;

 PORTD = 0b11101111;
 DDRD =  0b00011011;

 PORTC = 0b11111111;				//   
 DDRC =  0b00000000;

 PORTB = 0b11111011;	// 0b11111001 
 DDRB =  0b00010111;

 PORTA = 0b11111111;				//   
 DDRA =  0b00000000;

 
 //  TWI    I2C
 TWBR = 52;							//    TWI 100 
 TWDR = 0xFF;						//   
 TWCR = (1<<TWEN)|(0<<TWINT);		// Enable TWI,    TWI

 //     
 MCUCR = 0;
 EICRB = 0;				//    
 EIMSK = 0;				//   7

 sei();
}



int main(void)
{
 init_devices();									//  
 
 GreenLedOn();
 
 I2cBufer[0] = 0b10100000;
 I2cBufer[1] = 0x00;
 I2cBufer[2] = 0x19;
 I2cBufer[3] = 0x0A;
 
 I2cFrame = &I2cBufer[0];							//    
 I2cFrameLen = 3;									//   
 TWI_flags |=_BV(SendStart);						//     
 TWI_flags |=_BV(SendFrame);						//    
 TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)|(1<<TWIE);	//  

 while (TWI_flags & _BV(SendFrame))
 {
  if(TWI_flags & _BV(I2cError))						//    I2c
  {
   OrangeLedOn();									//   
   break;
  }//if(TWI_flags & _BV(I2cError))
 }//while (TWI_flags & _BV(SendFrame))
 
 I2cBufer[2] = 0x1A;

 I2cFrame = &I2cBufer[0];							//    
 I2cFrameLen = 3;									//   
 TWI_flags |=_BV(SendStart);						//    
 TWI_flags |=_BV(SendFrame);						//    
 TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)|(1<<TWIE);	//  

 while (TWI_flags & _BV(SendFrame))
 {
  if(TWI_flags & _BV(I2cError))						//    I2c
  {
   OrangeLedOn();									//   
   break;
  }//if(TWI_flags & _BV(I2cError))
 }//while (TWI_flags & _BV(SendFrame))


 do{}while(1);

}
